# Copyright (c) 2021-2025 Arista Networks, Inc.
# Use of this source code is governed by the MIT license
# that can be found in the LICENSE file.
"""jinja_variable_name_format_rule.py - Rule class to check that variable names only use underscores."""

from __future__ import annotations

import re
from typing import Any, ClassVar

from j2lint.linter.error import LinterError
from j2lint.linter.rule import Rule
from j2lint.utils import get_jinja_expressions


class JinjaVariableNameFormatRule(Rule):
    """Rule class to check that variable names only use underscores."""

    rule_id = "V2"
    description: ClassVar = "If variable is multi-words, underscore `_` should be used as a separator: '{{ my_variable_name }}'"
    short_description = "jinja-variable-format"
    severity = "LOW"

    regex = re.compile(r"[a-zA-Z0-9-_\"']+[-][a-zA-Z0-9-_\"']+")

    def __init__(self, ignore: bool = False, warn: list[Any] | None = None) -> None:
        super().__init__(ignore=ignore, warn=warn)

    def checktext(self, filename: str, text: str) -> list[LinterError]:
        """Not Implemented for this rule."""
        raise NotImplementedError

    def checkline(self, filename: str, line: str, line_no: int) -> list[LinterError]:
        """Check if the given line matches the error regex, which matches variables using `-` in their name.

        Parameters
        ----------
        filename
            The filename to which the line belongs.
        line
            The content of the line to check.
        line_no:
            The line number.

        Returns
        -------
        list[LinterError]
            The list of LinterError generated by this rule.
        """
        variables = get_jinja_expressions(line, blank_literals=True)
        all_matches = []
        for var in variables:
            var_matches = re.findall(self.regex, var)
            var_matches = [match for match in var_matches if ("'" not in match) and ('"' not in match)]
            all_matches.extend(var_matches)

        return [LinterError(line_no, line, filename, self, f"{self.description}: {match}") for match in all_matches]
